@@ -6,6 +6,9 @@
 # (see guidata/__init__.py for details)
 
 """
+dataset.qtitemwidgets
+=====================
+
 Widget factories used to edit data items
 (factory registration is done in guidata.dataset.qtwidgets)
 (data item types are defined in guidata.dataset.datatypes)
@@ -21,6 +24,7 @@ import os.path as osp
 import sys
 import numpy
 import collections
+import datetime
 
 try:
     # PyQt4 4.3.3 on Windows (static DLLs) with py2exe installed:
@@ -31,12 +35,12 @@ try:
 except ImportError:
     pass
 
-from guidata.qt.QtGui import (QIcon, QPixmap, QHBoxLayout, QGridLayout,
+from guidata.qt.QtGui import (QIcon, QPixmap, QHBoxLayout, QGridLayout, QColor,
                               QColorDialog, QPushButton, QLineEdit, QCheckBox,
-                              QComboBox, QWidget, QTabWidget, QGroupBox,
+                              QComboBox, QTabWidget, QGroupBox, QDateTimeEdit,
                               QLabel, QTextEdit, QFrame, QDateEdit, QSlider,
-                              QDateTimeEdit, QColor)
-from guidata.qt.QtCore import Qt, QObject, SIGNAL
+                              QRadioButton, QVBoxLayout)
+from guidata.qt.QtCore import Qt
 from guidata.qt.compat import getexistingdirectory
 try:
     from guidata.qt.QtCore import QStringList
@@ -61,13 +65,14 @@ from guidata.py3compat import to_text_string, is_text_string
 
 class AbstractDataSetWidget(object):
     """
-    Base class for 'widgets' handled by DataSetEditLayout and it's derived classes
+    Base class for 'widgets' handled by `DataSetEditLayout` and it's derived 
+    classes.
     
-    this is a generic representation of an input (or display) widget that
+    This is a generic representation of an input (or display) widget that
     has a label and one or more entry field.
     
-    DataSetEditLayout uses a registry of *Item to *Widget mapping in order to
-    automatically create a GUI for a DataSet structure
+    `DataSetEditLayout` uses a registry of *Item* to *Widget* mapping in order 
+    to automatically create a GUI for a `DataSet` structure
     """
     READ_ONLY = False
     def __init__(self, item, parent_layout):
@@ -239,8 +244,7 @@ class LineEditWidget(AbstractDataSetWidget):
             if item.check_item():
                 self.edit.setEnabled(False)
             self.edit.setToolTip(_("Value is forced to %d") % item.get_max())
-        QObject.connect(self.edit, SIGNAL("textChanged(QString)"),
-                        self.line_edit_changed)
+        self.edit.textChanged.connect(self.line_edit_changed)
 
     def get(self):
         """Override AbstractDataSetWidget method"""
@@ -257,7 +261,10 @@ class LineEditWidget(AbstractDataSetWidget):
             
     def line_edit_changed(self, qvalue):
         """QLineEdit validator"""
-        value = self.item.from_string(to_text_string(qvalue))
+        if qvalue is not None:
+            value = self.item.from_string(to_text_string(qvalue))
+        else:
+            value = None
         if not self.item.check_value(value):
             self.edit.setStyleSheet("background-color:rgb(255, 175, 90);")
         else:
@@ -299,7 +306,7 @@ class TextEditWidget(AbstractDataSetWidget):
             if item.check_item():
                 self.edit.setEnabled(False)
             self.edit.setToolTip(_("Value is forced to %d") % item.get_max())
-        QObject.connect(self.edit, SIGNAL("textChanged()"), self.text_changed)
+        self.edit.textChanged.connect(self.text_changed)
 
     def __get_text(self):
         """Get QTextEdit text, replacing UTF-8 EOL chars by os.linesep"""
@@ -347,14 +354,13 @@ class CheckBoxWidget(AbstractDataSetWidget):
         
         self.store = self.item.get_prop("display", "store", None)
         if self.store:
-            QWidget.connect(self.checkbox, SIGNAL("stateChanged(int)"),
-                            self.do_store)
+            self.checkbox.stateChanged.connect(self.do_store)
         
     def get(self):
         """Override AbstractDataSetWidget method"""
         value = self.item.get()
-        if value:
-            self.checkbox.setChecked(True)
+        if value is not None:
+            self.checkbox.setChecked(value)
     
     def set(self):
         """Override AbstractDataSetWidget method"""
@@ -391,6 +397,8 @@ class DateWidget(AbstractDataSetWidget):
         """Override AbstractDataSetWidget method"""
         value = self.item.get()
         if value:
+            if not isinstance(value, datetime.date):
+                value = datetime.date.fromordinal(value)
             self.dateedit.setDate(value)
     
     def set(self):
@@ -415,6 +423,8 @@ class DateTimeWidget(AbstractDataSetWidget):
         """Override AbstractDataSetWidget method"""
         value = self.item.get()
         if value:
+            if not isinstance(value, datetime.datetime):
+                value = datetime.datetime.fromtimestamp(value)
             self.dateedit.setDateTime(value)
     
     def set(self):
@@ -460,7 +470,7 @@ class ColorWidget(HLayoutMixin, LineEditWidget):
         super(ColorWidget, self).__init__(item, parent_layout)
         self.button = QPushButton("")
         self.button.setMaximumWidth(32)
-        QObject.connect(self.button, SIGNAL("clicked()"), self.select_color)
+        self.button.clicked.connect(self.select_color)
         self.group.addWidget(self.button)
         
     def update(self, value):
@@ -489,41 +499,62 @@ class ColorWidget(HLayoutMixin, LineEditWidget):
 
 class SliderWidget(HLayoutMixin, LineEditWidget):
     """
-    NumericTypeItem with Slider
+    IntItem with Slider
     """
+    DATA_TYPE = int
+
     def __init__(self, item, parent_layout):
         super(SliderWidget, self).__init__(item, parent_layout)
+        self.slider = self.vmin = self.vmax = None
         if item.get_prop_value("display", "slider"):
+            self.vmin = item.get_prop_value("data", "min")
+            self.vmax = item.get_prop_value("data", "max")
+            assert self.vmin is not None and self.vmax is not None, \
+                "SliderWidget requires that item min/max have been defined"
             self.slider = QSlider()
             self.slider.setOrientation(Qt.Horizontal)
-            vmin = item.get_prop_value("data", "min")
-            vmax = item.get_prop_value("data", "max")
-            vstep = item.get_prop_value("data", "step", 1)
-            assert vmin is not None and vmax is not None, "SliderWidget requires that NumericTypeItem min/max have been defined"
-            
-            self.slider.setRange(0, int((vmax-vmin)/vstep))
-            self.convert_value = lambda v: v*vstep + vmin
-            
-            QObject.connect(self.slider, SIGNAL("valueChanged(int)"),
-                            self.value_changed)
+            self.setup_slider(item)
+            self.slider.valueChanged.connect(self.value_changed)
             self.group.addWidget(self.slider)
-        else:
-            self.slider = None
+    
+    def value_to_slider(self, value):
+        return value
+    
+    def slider_to_value(self, value):
+        return value
+            
+    def setup_slider(self, item):
+        self.slider.setRange(self.vmin, self.vmax)
         
     def update(self, value):
         """Reimplement LineEditWidget method"""
         LineEditWidget.update(self, value)
-        if  self.slider is not None and isinstance(value, int):
+        if  self.slider is not None and isinstance(value, self.DATA_TYPE):
             self.slider.blockSignals(True)
-            self.slider.setValue(int(value))
+            self.slider.setValue(self.value_to_slider(value))
             self.slider.blockSignals(False)
             
     def value_changed(self, ivalue):
         """Update the lineedit"""
-        value = str(self.convert_value(ivalue))
+        value = str(self.slider_to_value(ivalue))
         self.edit.setText(value)
         self.update(value)
 
+class FloatSliderWidget(SliderWidget):
+    """
+    FloatItem with Slider
+    """
+    DATA_TYPE = float
+
+    def value_to_slider(self, value):
+        return (value-self.vmin)*100/(self.vmax-self.vmin)
+    
+    def slider_to_value(self, value):
+        return value*(self.vmax-self.vmin)/100+self.vmin
+            
+    def setup_slider(self, item):
+        self.slider.setRange(0, 100)
+
 
 def _get_child_title_func(ancestor):
     previous_ancestor = None
@@ -548,7 +579,7 @@ class FileWidget(HLayoutMixin, LineEditWidget):
         button = QPushButton()
         fmt = item.get_prop_value("data", "formats")
         button.setIcon(get_icon('%s.png' % fmt[0].lower(), default='file.png'))
-        QObject.connect(button, SIGNAL("clicked()"), self.select_file)
+        button.clicked.connect(self.select_file)
         self.group.addWidget(button)
         self.basedir = item.get_prop_value("data", "basedir")
         self.all_files_first = item.get_prop_value("data", "all_files_first")
@@ -593,7 +624,7 @@ class DirectoryWidget(HLayoutMixin, LineEditWidget):
         super(DirectoryWidget, self).__init__(item, parent_layout)
         button = QPushButton()
         button.setIcon(get_std_icon('DirOpenIcon'))
-        QObject.connect(button, SIGNAL("clicked()"), self.select_directory)
+        button.clicked.connect(self.select_directory)
         self.group.addWidget(button)
 
     def select_directory(self):
@@ -612,13 +643,19 @@ class ChoiceWidget(AbstractDataSetWidget):
     """
     def __init__(self, item, parent_layout):
         super(ChoiceWidget, self).__init__(item, parent_layout)
-        self.combobox = self.group = QComboBox()
-        self.combobox.setToolTip(item.get_help())
-        
-        self.__first_call = True
+        self._first_call = True
+        self.is_radio = item.get_prop_value("display", "radio")
         self.store = self.item.get_prop("display", "store", None)
-        QWidget.connect(self.combobox, SIGNAL("currentIndexChanged(int)"),
-                        self.index_changed)
+        if self.is_radio:
+            self.group = QGroupBox()
+            self.group.setToolTip(item.get_help())
+            self.vbox = QVBoxLayout()
+            self.group.setLayout(self.vbox)
+            self._buttons = []
+        else:
+            self.combobox = self.group = QComboBox()
+            self.combobox.setToolTip(item.get_help())
+            self.combobox.currentIndexChanged.connect(self.index_changed)
         
     def index_changed(self, index):
         if self.store:
@@ -633,12 +670,21 @@ class ChoiceWidget(AbstractDataSetWidget):
             cb(self.item.instance, self.item.item, self.value())
             self.parent_layout.update_widgets(except_this_one=self)
     
-    def fill_combo(self):
-        self.combobox.blockSignals(True)
-        while self.combobox.count():
-            self.combobox.removeItem(0)
+    def initialize_widget(self):
+        if self.is_radio:
+            for button in self._buttons:
+                button.toggled.disconnect(self.index_changed)
+                self.vbox.removeWidget(button)
+                button.deleteLater()
+            self._buttons = []
+        else:
+            self.combobox.blockSignals(True)
+            while self.combobox.count():
+                self.combobox.removeItem(0)
         _choices = self.item.get_prop_value("data", "choices")
         for key, lbl, img in _choices:
+            if self.is_radio:
+                button = QRadioButton(lbl, self.group)
             if img:
                 if is_text_string(img):
                     if not osp.isfile(img):
@@ -646,14 +692,42 @@ class ChoiceWidget(AbstractDataSetWidget):
                     img = QIcon(img)
                 elif isinstance(img, collections.Callable):
                     img = img(key)
-                self.combobox.addItem(img, lbl)
-            else:
+                if self.is_radio:
+                    button.setIcon(img)
+                else:
+                    self.combobox.addItem(img, lbl)
+            elif not self.is_radio:
                 self.combobox.addItem(lbl)
-        self.combobox.blockSignals(False)
-        
+            if self.is_radio:
+                self._buttons.append(button)
+                self.vbox.addWidget(button)
+                button.toggled.connect(self.index_changed)
+        if not self.is_radio:
+            self.combobox.blockSignals(False)
+
+    def set_widget_value(self, idx):
+        if self.is_radio:
+            for button in self._buttons:
+                button.blockSignals(True)
+            self._buttons[idx].setChecked(True)
+            for button in self._buttons:
+                button.blockSignals(False)
+        else:
+            self.combobox.blockSignals(True)
+            self.combobox.setCurrentIndex(idx)
+            self.combobox.blockSignals(False)
+    
+    def get_widget_value(self):
+        if self.is_radio:
+            for index, widget in enumerate(self._buttons):
+                if widget.isChecked():
+                    return index
+        else:
+            return self.combobox.currentIndex()
+
     def get(self):
         """Override AbstractDataSetWidget method"""
-        self.fill_combo()
+        self.initialize_widget()
         value = self.item.get()
         if value is not None:
             idx = 0
@@ -662,12 +736,10 @@ class ChoiceWidget(AbstractDataSetWidget):
                 if key == value:
                     break
                 idx += 1
-            self.combobox.blockSignals(True)
-            self.combobox.setCurrentIndex(idx)
-            self.combobox.blockSignals(False)
-            if self.__first_call:
+            self.set_widget_value(idx)
+            if self._first_call:
                 self.index_changed(idx)
-                self.__first_call = False
+                self._first_call = False
         
     def set(self):
         """Override AbstractDataSetWidget method"""
@@ -678,7 +750,7 @@ class ChoiceWidget(AbstractDataSetWidget):
         self.item.set(value)
 
     def value(self):
-        index = self.combobox.currentIndex()
+        index = self.get_widget_value()
         choices = self.item.get_prop_value("data", "choices")
         return choices[index][0]
         
@@ -761,7 +833,7 @@ class FloatArrayWidget(AbstractDataSetWidget):
                                                  _("Largest element in array"))
         self.layout.addLayout(self.max_line, 2, 0)
         
-        QObject.connect(edit_button, SIGNAL("clicked()"), self.edit_array )
+        edit_button.clicked.connect(self.edit_array)
         self.arr = None # le tableau si il a été modifié
         self.instance = None
 
@@ -769,8 +841,13 @@ class FloatArrayWidget(AbstractDataSetWidget):
         """Open an array editor dialog"""
         parent = self.parent_layout.parent
         label = self.item.get_prop_value("display", "label")
-        from spyderlib.widgets.arrayeditor import ArrayEditor
-        editor = ArrayEditor(parent)
+        try:
+            # Spyder 3
+            from spyderlib.widgets.variableexplorer import arrayeditor
+        except ImportError:
+            # Spyder 2
+            from spyderlib.widgets import arrayeditor
+        editor = arrayeditor.ArrayEditor(parent)
         if editor.setup_and_check(self.arr, title=label):
             if editor.exec_():
                 self.update(self.arr)
@@ -842,7 +919,7 @@ class ButtonWidget(AbstractDataSetWidget):
             if is_text_string(_icon):
                 _icon = get_icon(_icon)
             self.button.setIcon(_icon)
-        QObject.connect(self.button, SIGNAL("clicked()"), self.clicked)
+        self.button.clicked.connect(self.clicked)
         self.cb_value = None
 
     def get(self):
