@@ -30,6 +30,7 @@ import warnings
 
 # Local imports
 from guidata.configtools import get_module_path
+from guidata.py3compat import to_binary_string
 
 
 #==============================================================================
@@ -104,8 +105,9 @@ def _remove_later(fname):
 def get_msvc_version(python_version):
     """Return Microsoft Visual C++ version used to build this Python version"""
     if python_version is None:
-        python_version = '2.7'
-        warnings.warn("assuming Python 2.7 target")
+        python_version = '%s.%s' % (sys.version_info.major,
+                                    sys.version_info.minor)
+        warnings.warn("Assuming Python %s target" % python_version)
     if python_version in ('2.6', '2.7', '3.0', '3.1', '3.2'):
         # Python 2.6-2.7, 3.0-3.2 were built with Visual Studio 9.0.21022.8
         # (i.e. Visual C++ 2008, not Visual C++ 2008 SP1!)
@@ -117,7 +119,23 @@ def get_msvc_version(python_version):
     else:
         raise RuntimeError("Unsupported Python version %s" % python_version)
 
-def get_msvc_dlls(msvc_version, architecture=None):
+def get_dll_architecture(path):
+    """Return DLL architecture (32 or 64bit) using Microsoft dumpbin.exe"""
+    os.environ['PATH'] += r';C:\Program Files (x86)\Microsoft Visual Studio 9.0\Common7\IDE\;C:\Program Files (x86)\Microsoft Visual Studio 9.0\VC\BIN;C:\Program Files (x86)\Microsoft Visual Studio 10.0\Common7\IDE\;C:\Program Files (x86)\Microsoft Visual Studio 10.0\VC\BIN'
+    process = Popen(['dumpbin', '/HEADERS', osp.basename(path)], stdout=PIPE,
+                     stderr=PIPE, cwd=osp.dirname(path), shell=True)
+    output = process.stdout.read()
+    error = process.stderr.read()
+    if error:
+        raise RuntimeError(error)
+    elif 'x86' in output:
+        return 32
+    elif 'x64' in output:
+        return 64
+    else:
+        raise ValueError('Unable to get DLL architecture')
+
+def get_msvc_dlls(msvc_version, architecture=None, check_architecture=False):
     """Get the list of Microsoft Visual C++ DLLs associated to 
     architecture and Python version, create the manifest file.
     
@@ -126,6 +144,7 @@ def get_msvc_dlls(msvc_version, architecture=None):
     current_architecture = 64 if sys.maxsize > 2**32 else 32
     if architecture is None:
         architecture = current_architecture
+    assert architecture in (32, 64)
 
     filelist = []
 
@@ -196,14 +215,22 @@ def get_msvc_dlls(msvc_version, architecture=None):
         # applications and is included on all 64-bit versions of Windows
         # (source: http://en.wikipedia.org/wiki/WoW64)
         #
-        # In other words, "SysWOW64" contains 64-bit DLL and applications, 
+        # In other words, "SysWOW64" contains 32-bit DLL and applications, 
         # whereas "System32" contains 64-bit DLL and applications on a 64-bit 
         # system.
-        sysdir = "System32"
-        if not is_64bit_windows and architecture == 64:
-            raise RuntimeError("Can't find 64-bit MSVC DLLs on a 32-bit OS")
-        if is_64bit_windows and architecture == 32:
-            sysdir = "SysWOW64"
+        if architecture == 64:
+            # 64-bit DLLs are located in...
+            if is_64bit_windows:
+                sysdir = "System32"  # on a 64-bit OS
+            else:
+                # ...no directory to be found!
+                raise RuntimeError("Can't find 64-bit DLLs on a 32-bit OS")
+        else:
+            # 32-bit DLLs are located in...
+            if is_64bit_windows:
+                sysdir = "SysWOW64"  # on a 64-bit OS
+            else:
+                sysdir = "System32"  # on a 32-bit OS
 
         for dllname in namelist:
             fname = osp.join(windir, sysdir, dllname)
@@ -215,9 +242,21 @@ def get_msvc_dlls(msvc_version, architecture=None):
 
     else:
         raise RuntimeError("Unsupported MSVC version %s" % msvc_version)
-    
+
+    if check_architecture:
+        for path in filelist:
+            if path.endswith('.dll'):
+                try:
+                    arch = get_dll_architecture(path)
+                except RuntimeError:
+                    return
+                if arch != architecture:
+                    raise RuntimeError("%s: expecting %dbit, found %dbit"\
+                                       % (path, architecture, arch))
+
     return filelist
 
+
 def create_msvc_data_files(architecture=None, python_version=None,
                            verbose=False):
     """Including Microsoft Visual C++ DLLs"""
@@ -283,6 +322,10 @@ class Distribution(object):
                         'PIL.ImageTk', 'FixTk', 'bsddb', 'email',
                         'pywin.debugger', 'pywin.debugger.dbgcon',
                         'matplotlib']
+    if sys.version_info.major == 2:
+        #  Fixes compatibility issue with IPython (more specifically with one 
+        #  of its dependencies: `jsonschema`) on Python 2.7
+        DEFAULT_EXCLUDES += ['collections.abc']
     DEFAULT_INCLUDES = []
     DEFAULT_BIN_EXCLUDES = ['MSVCP100.dll', 'MSVCP90.dll', 'w9xpopen.exe',
                             'MSVCP80.dll', 'MSVCR80.dll']
@@ -306,7 +349,7 @@ class Distribution(object):
         self.bin_path_excludes = self.DEFAULT_BIN_PATH_EXCLUDES
         self.msvc = os.name == 'nt'
         self._py2exe_is_loaded = False
-        self._pyqt4_added = False
+        self._pyqt_added = False
         self._pyside_added = False
         # Attributes relative to cx_Freeze:
         self.executables = []
@@ -372,7 +415,7 @@ class Distribution(object):
     def add_text_data_file(self, filename, contents):
         """Create temporary data file *filename* with *contents*
         and add it to *data_files*"""
-        open(filename, 'wb').write(contents)
+        open(filename, 'wb').write(to_binary_string(contents))
         self.data_files += [("", (filename, ))]
         _remove_later(filename)
     
@@ -380,18 +423,25 @@ class Distribution(object):
         self.data_files += [(destdir, (filename, ))]
 
     #------ Adding packages
-    def add_pyqt4(self):
-        """Include module PyQt4 to the distribution"""
-        if self._pyqt4_added:
+    def add_pyqt(self):
+        """Include module PyQt4 or PyQt5 to the distribution"""
+        if self._pyqt_added:
             return
-        self._pyqt4_added = True
+        self._pyqt_added = True
         
-        self.includes += ['sip', 'PyQt4.Qt', 'PyQt4.QtSvg', 'PyQt4.QtNetwork']
+        try:
+            import PyQt4 as PyQt
+            qtver = 4
+        except ImportError:
+            import PyQt5 as PyQt
+            qtver = 5
+        self.includes += ['sip', 'PyQt%d.Qt' % qtver,
+                          'PyQt%d.QtSvg' % qtver,
+                          'PyQt%d.QtNetwork' % qtver]
         
-        import PyQt4
-        pyqt_path = osp.dirname(PyQt4.__file__)
+        pyqt_path = osp.dirname(PyQt.__file__)
         
-        # Configuring PyQt4
+        # Configuring PyQt
         conf = os.linesep.join(["[Paths]", "Prefix = .", "Binaries = ."])
         self.add_text_data_file('qt.conf', conf)
         
@@ -403,15 +453,18 @@ class Distribution(object):
                 shutil.rmtree(pyqt_tmp)
             os.mkdir(pyqt_tmp)
             vc90man_pyqt = osp.join(pyqt_tmp, vc90man)
-            man = open(vc90man, "r").read().replace('<file name="',
-                                        '<file name="Microsoft.VC90.CRT\\')
-            open(vc90man_pyqt, 'w').write(man)
+            if osp.isfile(vc90man):
+                man = open(vc90man, "r").read().replace('<file name="',
+                                            '<file name="Microsoft.VC90.CRT\\')
+                open(vc90man_pyqt, 'w').write(man)
+            else:
+                vc90man_pyqt = None
         for dirpath, _, filenames in os.walk(osp.join(pyqt_path,
                                                       "plugins")):
             filelist = [osp.join(dirpath, f) for f in filenames
                         if osp.splitext(f)[1] in ('.dll', '.py')]
-            if self.msvc and [f for f in filelist
-                           if osp.splitext(f)[1] == '.dll']:
+            if self.msvc and vc90man_pyqt is not None and\
+               [f for f in filelist if osp.splitext(f)[1] == '.dll']:
                 # Where there is a DLL build with Microsoft Visual C++ 2008,
                 # there must be a manifest file as well...
                 # ...congrats to Microsoft for this great simplification!
@@ -485,15 +538,19 @@ class Distribution(object):
     def add_qt_bindings(self):
         """Include Qt bindings, i.e. PyQt4 or PySide"""
         try:
-            imp.find_module('PyQt4')
-            self.add_modules('PyQt4')
+            imp.find_module('PyQt5')
+            self.add_modules('PyQt5')
         except ImportError:
-            self.add_modules('PySide')
+            try:
+                imp.find_module('PyQt4')
+                self.add_modules('PyQt4')
+            except ImportError:
+                self.add_modules('PySide')
 
     def add_matplotlib(self):
         """Include module Matplotlib to the distribution"""
         if 'matplotlib' in self.excludes:
-            self.excludes.pop(self.excludes.index('matplotlib'))
+            self.excludes.remove('matplotlib')
         try:
             import matplotlib.numerix  # analysis:ignore
             self.includes += ['matplotlib.numerix.ma',
@@ -511,20 +568,16 @@ class Distribution(object):
         """Include module *module_name*"""
         for module_name in module_names:
             print("Configuring module '%s'" % module_name)
-            if module_name == 'PyQt4':
-                self.add_pyqt4()
+            if module_name in ('PyQt4', 'PyQt5'):
+                self.add_pyqt()
             elif module_name == 'PySide':
                 self.add_pyside()
-            elif module_name == 'scipy.io':
-                self.includes += ['scipy.io.matlab.streams']
+            elif module_name == 'scipy':
+                self.add_module_dir('scipy')
             elif module_name == 'matplotlib':
                 self.add_matplotlib()
             elif module_name == 'h5py':
-                import h5py
-                for attr in ['_stub', '_sync', 'utils', '_conv', '_proxy',
-                             'defs']:
-                    if hasattr(h5py, attr):
-                        self.includes.append('h5py.%s' % attr)
+                self.add_module_dir('h5py')
                 if self.bin_path_excludes is not None and os.name == 'nt':
                     # Specific to cx_Freeze on Windows: avoid including a zlib dll
                     # built with another version of Microsoft Visual Studio
@@ -558,11 +611,7 @@ class Distribution(object):
             elif module_name == 'guidata':
                 self.add_module_data_files('guidata', ("images", ),
                                        ('.png', '.svg'), copy_to_root=False)
-                try:
-                    imp.find_module('PyQt4')
-                    self.add_pyqt4()
-                except ImportError:
-                    self.add_pyside()
+                self.add_qt_bindings()
             elif module_name == 'guiqwt':
                 self.add_module_data_files('guiqwt', ("images", ),
                                        ('.png', '.svg'), copy_to_root=False)
@@ -604,6 +653,34 @@ class Distribution(object):
                 for name in pathlist:
                     print("  ", name)
     
+    def add_module_dir(self, module_name, verbose=False, exclude_dirs=[]):
+        """
+        Collect all module files for module *module_name*
+        and add them to *data_files*
+        """
+        module_dir = get_module_path(module_name)
+        nstrip = len(module_dir) + len(osp.sep)
+        for dirpath, dirnames, filenames in os.walk(module_dir):
+            if osp.basename(dirpath) in exclude_dirs:
+                continue
+            for dn in dirnames[:]:
+                if not osp.isfile(osp.join(dirpath, dn, '__init__.py')):
+                    dirnames.remove(dn)
+            dirname = osp.join(module_name, dirpath[nstrip:])
+            for filename in filenames:
+                ext = osp.splitext(filename)[1].lower()
+                if ext in ('.py', '.pyd'):
+                    if filename == '__init__.py':
+                        fn = dirname
+                    else:
+                        fn = osp.splitext(osp.join(dirname, filename))[0]
+                    if fn.endswith(os.sep):
+                        fn = fn[:-1]
+                    modname = ".".join(fn.split(os.sep))
+                    self.includes += [modname]
+                    if verbose:
+                        print("  + ", modname)
+    
     def add_module_data_files(self, module_name, data_dir_names, extensions,
                               copy_to_root=True, verbose=False,
                               exclude_dirs=[]):
@@ -723,9 +800,36 @@ class Distribution(object):
         assert not self._py2exe_is_loaded, \
                "cx_Freeze can't be executed after py2exe"
         from cx_Freeze import setup
+        
+        #===== Monkey-patching cx_Freeze (backported from v5.0 dev) ===========
+        from cx_Freeze import hooks
+        def load_h5py(finder, module):
+            """h5py module has a number of implicit imports"""
+            finder.IncludeModule('h5py.defs')
+            finder.IncludeModule('h5py.utils')
+            finder.IncludeModule('h5py._proxy')
+            try:
+                import h5py.api_gen
+                finder.IncludeModule('h5py.api_gen')
+            except ImportError:
+                pass
+            finder.IncludeModule('h5py._errors')
+            finder.IncludeModule('h5py.h5ac')
+        hooks.load_h5py = load_h5py
+        #===== Monkey-patching cx_Freeze (backported from v5.0 dev) ===========
+
+        #===== Monkey-patching cx_Freeze for Scipy ============================
+        def load_scipy(finder, module):
+            pass
+        hooks.load_scipy = load_scipy
+        #===== Monkey-patching cx_Freeze for Scipy ============================
+
         if cleanup:
             self.__cleanup()
         sys.argv += ["build"]
+        excv = "3" if sys.version[0] == "2" else "2"
+        self.excludes += ["sympy.mpmath.libmp.exec_py%s" % excv]
+        self.excludes += ["PyQt4.uic.port_v%s" % excv]
         build_exe = dict(include_files=to_include_files(self.data_files),
                          includes=self.includes, excludes=self.excludes,
                          bin_excludes=self.bin_excludes,
@@ -738,3 +842,15 @@ class Distribution(object):
               options=dict(build_exe=build_exe))
         if create_archive:
             self.__create_archive(create_archive)
+
+
+if __name__ == '__main__':
+    for python_version in ('2.7', '3.3'):
+        for arch in (32, 64):
+            print('Python %s %dbit' % (python_version, arch))
+            msvc_version = get_msvc_version(python_version)
+            filelist = get_msvc_dlls(msvc_version, architecture=arch)
+            for fname in filelist:
+                if '.dll' in fname:
+                    print(get_dll_architecture(fname))
+            print()
