@@ -14,13 +14,16 @@ writing a complete setup script.
 
 SHOW = True # Show test in GUI-based test launcher
 
+import os.path as osp
+
 from guidata.disthelpers import Distribution
 
 if __name__ == '__main__':
     dist = Distribution()
     dist.setup(name="Application demo", version='1.0.0',
                description="Application demo based on editgroupbox.py",
-               script="editgroupbox.py", target_name="demo.exe")
+               script=osp.join(osp.dirname(__file__), "editgroupbox.py"),
+               target_name="demo.exe")
     dist.add_modules('guidata')
     dist.build('cx_Freeze')
 
